#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Copyright  2007 Bill Menees.  All rights reserved.
	Bill@Menees.com

	$History: BaseConverterControl.cs $
	
	*****************  Version 1  *****************
	User: Bill         Date: 11/28/07   Time: 9:23p
	Created in $/CSharp/MeneesVSTools/2008
	
	*****************  Version 1  *****************
	User: Bill         Date: 3/12/07    Time: 10:50a
	Created in $/CSharp/MeneesVSTools/2005
	Updated to v1.1.

-----------------------------------------------------------------------------*/

#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

#endregion

namespace Menees.VSTools
{
	public partial class BaseConverterControl : UserControl
	{
		#region Constructors

		public BaseConverterControl()
		{
			InitializeComponent();

			m_currentTextBox = txtHex;
			txtHex.Tag = NumberBase.Hex;
			txtDecimal.Tag = NumberBase.Decimal;
			txtBinary.Tag = NumberBase.Binary;

			CurrentByteOrder = GetEnumValue(Settings.Default.BaseConverterByteOrder, ByteOrder.Numeric);
			CurrentDataType = GetEnumValue(Settings.Default.BaseConverterDataType, DataType.Int);

			Settings.Default.PropertyChanged += new PropertyChangedEventHandler(Settings_PropertyChanged);
		}

		#endregion

		#region Private Properties

		private ByteOrder CurrentByteOrder
		{
			get
			{
				return (ByteOrder)cbByteOrder.SelectedIndex;
			}
			set
			{
				cbByteOrder.SelectedIndex = (int)value;
			}
		}

		private DataType CurrentDataType
		{
			get
			{
				return (DataType)cbDataType.SelectedIndex;
			}
			set
			{
				cbDataType.SelectedIndex = (int)value;
			}
		}

		#endregion

		#region Private Event Handlers

		private void cbByteOrder_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (CurrentByteOrder != (ByteOrder)Settings.Default.BaseConverterByteOrder)
			{
				Settings.Default.BaseConverterByteOrder = (int)CurrentByteOrder;
				UpdateFromCurrentTextBox();
			}
		}

		private void cbDataType_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (CurrentDataType != (DataType)Settings.Default.BaseConverterDataType)
			{
				Settings.Default.BaseConverterDataType = (int)CurrentDataType;
				UpdateFromCurrentTextBox();
			}
		}

		private void txtHex_TextChanged(object sender, EventArgs e)
		{
			if (!m_updatingDisplay)
			{
				SetCurrentTextBox(txtHex);
			}
		}

		private void txtDecimal_TextChanged(object sender, EventArgs e)
		{
			if (!m_updatingDisplay)
			{
				SetCurrentTextBox(txtDecimal);
			}
		}

		private void txtBinary_TextChanged(object sender, EventArgs e)
		{
			if (!m_updatingDisplay)
			{
				SetCurrentTextBox(txtBinary);
			}
		}

		void Settings_PropertyChanged(object sender, PropertyChangedEventArgs e)
		{
			if (e.PropertyName == "NumberTrimLeadingZerosNumeric" ||
				e.PropertyName == "NumberTrimLeadingZerosEndian" ||
				e.PropertyName == "NumberUseSpaceSeparators"  ||
				e.PropertyName == "NumberUseSystemNumberFmtForDecimal")
			{
				UpdateFromCurrentTextBox();
			}
		}

		#endregion

		#region Private Helper Methods

		private T GetEnumValue<T>(int value, T defaultValue) where T : struct
		{
			if (Enum.IsDefined(typeof(T), value))
			{
				return (T)Enum.ToObject(typeof(T), value);
			}
			else
			{
				return defaultValue;
			}
		}

		private void UpdateDisplay(Number num, TextBox txtSource)
		{
			bool previouslyUpdatingDisplay = m_updatingDisplay;
			m_updatingDisplay = true;
			try
			{
				if (txtHex != txtSource)
				{
					txtHex.Text = num.HexValue;
				}

				if (txtDecimal != txtSource)
				{
					txtDecimal.Text = num.DecimalValue;
				}

				if (txtBinary != txtSource)
				{
					txtBinary.Text = num.BinaryValue;
				}
			}
			finally
			{
				m_updatingDisplay = previouslyUpdatingDisplay;
			}
		}

		private void SetCurrentTextBox(TextBox textBox)
		{
			m_currentTextBox = textBox;
			UpdateFromCurrentTextBox();
		}

		private void UpdateFromCurrentTextBox()
		{
			NumberBase numBase = GetNumberBase(m_currentTextBox);
			Number num = new Number(m_currentTextBox.Text, numBase, CurrentByteOrder, CurrentDataType);
			UpdateDisplay(num, m_currentTextBox);
		}

		private NumberBase GetNumberBase(TextBox textBox)
		{
			return (NumberBase)textBox.Tag;
		}

		#endregion

		#region Private Data Members

		private TextBox m_currentTextBox;
		private bool m_updatingDisplay;

		private static readonly List<char> s_binaryDigits = new List<char>(new char[] { '0', '1' });
		private static readonly List<char> s_decimalDigits = new List<char>(new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9'});
		private static readonly List<char> s_hexDigits = new List<char>(new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'});

		#endregion
	}
}
