#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Copyright (c) 2005 Bill Menees.  All rights reserved.
	Bill@Menees.com

	$History: Connect.cs $
	
	*****************  Version 1  *****************
	User: Bill         Date: 11/28/07   Time: 9:23p
	Created in $/CSharp/MeneesVSTools/2008
	
	*****************  Version 5  *****************
	User: Bill         Date: 11/21/07   Time: 9:05p
	Updated in $/CSharp/MeneesVSTools/2005
	Recover from VSS crash.
	
	*****************  Version 3  *****************
	User: Bill         Date: 5/08/05    Time: 11:11a
	Updated in $/CSharp/MeneesVSTools/2005
	
	*****************  Version 2  *****************
	User: Bill         Date: 5/08/05    Time: 9:33a
	Updated in $/CSharp/MeneesVSTools/2005
	
	*****************  Version 1  *****************
	User: Bill         Date: 5/01/05    Time: 9:57a
	Created in $/CSharp/MeneesVSTools/2005

-----------------------------------------------------------------------------*/

#endregion

#region	Using Directives

using System;
using System.Windows.Forms;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Extensibility;
using EnvDTE;
using EnvDTE80;
using Microsoft.VisualStudio.CommandBars;
using System.Resources;
using System.Globalization;
using System.Reflection;

#endregion

namespace Menees.VSTools
{
	/// <summary>The object for implementing an Add-in.</summary>
	/// <seealso class='IDTExtensibility2' />
	public class Connect : Object, IDTExtensibility2, IDTCommandTarget
	{
		#region Constructors

		/// <summary>Implements the constructor for the Add-in object. Place your initialization code within this method.</summary>
		public Connect()
		{
		}
		
		#endregion

		#region IDTExtensibility2

		/// <summary>Implements the OnConnection method of the IDTExtensibility2 interface. Receives notification that the Add-in is being loaded.</summary>
		/// <param term='application'>Root object of the host application.</param>
		/// <param term='connectMode'>Describes how the Add-in is being loaded.</param>
		/// <param term='addInInst'>Object representing this Add-in.</param>
		/// <seealso class='IDTExtensibility2' />
		public void OnConnection(object application, ext_ConnectMode connectMode, object addInInst, ref Array custom)
		{
			m_Application = (DTE2)application;
			m_AddIn = (AddIn)addInInst;
			m_strCommandPrefix = m_AddIn.ProgID + ".";
			m_CommandProcessor = new CommandProcessor(m_Application, m_AddIn);

			//Note: To get the UISetup mode to fire again, you need to delete the value
			//for this add-in (Menees.VSTools.Connect;...) from the registry key:
			//HKEY_CURRENT_USER\Software\Microsoft\VisualStudio\9.0\PreloadAddinStateManaged.
			//
			//Or you can pass the following arguments to DevEnv.exe on startup:
			//		/resetaddin Menees.VSTools.Connect
			if (connectMode == ext_ConnectMode.ext_cm_UISetup)
			{
				//Clean up any old UI setup first.  This gets rid of old commands and toolbar buttons.
				//It returns a reference to the existing toolbar though so GUI positioning isn't lost.
				CommandBar Toolbar = CleanUpCurrentUISetup();

				//Create a new toolbar for this add-in if necessary.
				if (Toolbar == null)
				{
					Commands2 Cmds = (Commands2)m_Application.Commands;
					Toolbar = (CommandBar)Cmds.AddCommandBar(c_strToolbarName, vsCommandBarType.vsCommandBarTypeToolbar, null, 0);
					//Make sure the toolbar is docked.
					Toolbar.Position = MsoBarPosition.msoBarTop;
				}

				//Make sure the toolbar is visible.  If we deleted all the controls
				//from an existing toolbar, it may not be visible.
				Toolbar.Visible = true;

				try
				{
					//Add each named command and put it on the toolbar.
					int iNumCmds = s_arCommandInfo.Length;
					for (int i = 0; i < iNumCmds; i++)
					{
						CommandInfo Cmd = s_arCommandInfo[i];
						AddCommand(Cmd, Toolbar, i);
					}
				}
				catch (System.Exception e)
				{
					MessageBox.Show(e.Message, c_strToolbarName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				}
			}
			else if (connectMode == ext_ConnectMode.ext_cm_Startup || connectMode == ext_ConnectMode.ext_cm_AfterStartup)
			{
				m_CommandProcessor.CreateToolWindows();
			}
		}

		/// <summary>Implements the OnDisconnection method of the IDTExtensibility2 interface. Receives notification that the Add-in is being unloaded.</summary>
		/// <param term='disconnectMode'>Describes how the Add-in is being unloaded.</param>
		/// <param term='custom'>Array of parameters that are host application specific.</param>
		/// <seealso class='IDTExtensibility2' />
		public void OnDisconnection(ext_DisconnectMode disconnectMode, ref Array custom)
		{
			if (disconnectMode == ext_DisconnectMode.ext_dm_UserClosed ||
				disconnectMode == ext_DisconnectMode.ext_dm_HostShutdown)
			{
				if (m_CommandProcessor != null)
				{
					m_CommandProcessor.Dispose();
					m_CommandProcessor = null;
				}

				//Make sure the user's settings get saved out.  Do this last in case the command processor
				//or its helper objects need to save settings (e.g., tool window state data).
				Settings.Default.Save();
			}
		}

		/// <summary>Implements the OnAddInsUpdate method of the IDTExtensibility2 interface. Receives notification when the collection of Add-ins has changed.</summary>
		/// <param term='custom'>Array of parameters that are host application specific.</param>
		/// <seealso class='IDTExtensibility2' />		
		public void OnAddInsUpdate(ref Array custom)
		{
		}

		/// <summary>Implements the OnStartupComplete method of the IDTExtensibility2 interface. Receives notification that the host application has completed loading.</summary>
		/// <param term='custom'>Array of parameters that are host application specific.</param>
		/// <seealso class='IDTExtensibility2' />
		public void OnStartupComplete(ref Array custom)
		{
		}

		/// <summary>Implements the OnBeginShutdown method of the IDTExtensibility2 interface. Receives notification that the host application is being unloaded.</summary>
		/// <param term='custom'>Array of parameters that are host application specific.</param>
		/// <seealso class='IDTExtensibility2' />
		public void OnBeginShutdown(ref Array custom)
		{
		}
		
		#endregion		

		#region IDTCommandTarget

		/// <summary>Implements the QueryStatus method of the IDTCommandTarget interface. This is called when the command's availability is updated</summary>
		/// <param term='commandName'>The name of the command to determine state for.</param>
		/// <param term='neededText'>Text that is needed for the command.</param>
		/// <param term='status'>The state of the command in the user interface.</param>
		/// <param term='commandText'>Text requested by the neededText parameter.</param>
		/// <seealso class='Exec' />
		public void QueryStatus(string commandName, vsCommandStatusTextWanted neededText, ref vsCommandStatus status, ref object commandText)
		{
			status = vsCommandStatus.vsCommandStatusUnsupported;

			if (neededText == vsCommandStatusTextWanted.vsCommandStatusTextWantedNone && m_CommandProcessor != null)
			{
				string strInternalName = GetInternalCommandName(commandName);
				if (m_CommandProcessor.CanExecute(strInternalName))
				{
					status = (vsCommandStatus)(vsCommandStatus.vsCommandStatusSupported | vsCommandStatus.vsCommandStatusEnabled);
				}
			}
		}

		/// <summary>Implements the Exec method of the IDTCommandTarget interface. This is called when the command is invoked.</summary>
		/// <param term='commandName'>The name of the command to execute.</param>
		/// <param term='executeOption'>Describes how the command should be run.</param>
		/// <param term='varIn'>Parameters passed from the caller to the command handler.</param>
		/// <param term='varOut'>Parameters passed from the command handler to the caller.</param>
		/// <param term='bHandled'>Informs the caller if the command was handled or not.</param>
		/// <seealso class='Exec' />
		public void Exec(string commandName, vsCommandExecOption executeOption, ref object varIn, ref object varOut, ref bool bHandled)
		{
			bHandled = false;
			if (executeOption == vsCommandExecOption.vsCommandExecOptionDoDefault && m_CommandProcessor != null)
			{
				string strInternalName = GetInternalCommandName(commandName);
				bHandled = m_CommandProcessor.Execute(strInternalName);
			}
		}
		
		#endregion

		#region Private Methods

		private CommandBar CleanUpCurrentUISetup()
		{
			Commands2 Cmds = (Commands2)m_Application.Commands;

			//Before we add any commands, we need to delete any existing ones.
			foreach (Command Cmd in Cmds)
			{
				string strName = Cmd.Name;
				if (strName.StartsWith(m_strCommandPrefix))
				{
					Cmd.Delete();
				}
			}

			//Now we need to clean up any existing toolbar.
			CommandBars Bars = (CommandBars)m_Application.CommandBars;

			//Get rid of any old toolbar controls for this add-in.
			CommandBar Toolbar = null;
			foreach (CommandBar Bar in Bars)
			{
				if (Bar.Name == c_strToolbarName)
				{
					foreach (CommandBarControl Ctrl in Bar.Controls)
					{
						Ctrl.Delete(false);
					}

					Toolbar = Bar;
					break;
				}
			}

			//Get rid of any old menu items.
			foreach (CommandInfo info in s_arCommandInfo)
			{
				MenuCommandInfo menuInfo = info as MenuCommandInfo;
				if (menuInfo != null)
				{
					CommandBar menuBar = GetMenuPopupCommandBar(menuInfo);
					foreach (CommandBarControl Ctrl in menuBar.Controls)
					{
						if (Ctrl.Caption == menuInfo.Name)
						{
							Ctrl.Delete(false);
						}
					}
				}
			}

			return Toolbar;
		}

		/// <summary>
		/// This strips off the ProgId prefix and returns the local command name.
		/// </summary>
		/// <param name="commandName">The full name of the command.</param>
		/// <returns></returns>
		private string GetInternalCommandName(string strCmdName)
		{
			string strResult = "";

			if (strCmdName.StartsWith(m_strCommandPrefix))
			{
				strResult = strCmdName.Substring(m_strCommandPrefix.Length);
			}

			return strResult;
		}

		private Command AddCommand(CommandInfo Info, CommandBar Toolbar, int iCmdNum)
		{
			object[] arContextGUIDS = new object[] { };
			Commands2 commands = (Commands2)m_Application.Commands;

			MenuCommandInfo menuInfo = Info as MenuCommandInfo;
			vsCommandStyle eCmdStyle = menuInfo != null ? vsCommandStyle.vsCommandStylePictAndText : vsCommandStyle.vsCommandStylePict;

			//Add the named command.
			Command command = commands.AddNamedCommand2(m_AddIn, Info.Name, Info.Tooltip, Info.Tooltip,
				true, Info.ImageIndex, ref arContextGUIDS, (int)vsCommandStatus.vsCommandStatusSupported + (int)vsCommandStatus.vsCommandStatusEnabled,
				(int)eCmdStyle, vsCommandControlType.vsCommandControlTypeButton);

			if (menuInfo != null)
			{
				Toolbar = GetMenuPopupCommandBar(menuInfo);
				iCmdNum = 0;
			}

			//Add 1 to the command number because the control position is 1-based.
			//Note: If I passed in iCmdNum+1 directly, I kept getting E_UNEXPECTED.  But when I increment the
			//local variable, I can pass it in with no problems.  Very weird.
			iCmdNum += 1;
			CommandBarControl commandBarControl = (CommandBarControl)command.AddControl(Toolbar, iCmdNum);

			return command;
		}

		private CommandBar GetMenuPopupCommandBar(MenuCommandInfo menuInfo)
		{
			//Find the MenuBar command bar, which is the top-level command bar holding all the main menu items:
			CommandBar menuBarCommandBar = ((CommandBars)m_Application.CommandBars)["MenuBar"];

			//Find the correct command bar on the MenuBar command bar:
			string localMenuName = GetLocalMenuName(menuInfo.MenuName);
			CommandBarControl menuControl = menuBarCommandBar.Controls[localMenuName];
			CommandBarPopup menuPopup = (CommandBarPopup)menuControl;
			CommandBar namedMenuCommandBar = menuPopup.CommandBar;
			return namedMenuCommandBar;
		}

		private string GetLocalMenuName(string englishMenuName)
		{
			string result = englishMenuName;

			try
			{
				//This code will take the culture, append on the name of the menu then lookup that name
				//in the list of all the top-level menus in the file CommandBar.resx.
				ResourceManager resourceManager = new ResourceManager("Menees.VSTools.CommandBar", Assembly.GetExecutingAssembly());
				CultureInfo cultureInfo = new CultureInfo(m_Application.LocaleID);

				string resourceName;
				if (cultureInfo.TwoLetterISOLanguageName == "zh")
				{
					CultureInfo parentCultureInfo = cultureInfo.Parent;
					resourceName = string.Concat(parentCultureInfo.Name, englishMenuName);
				}
				else
				{
					resourceName = string.Concat(cultureInfo.TwoLetterISOLanguageName, englishMenuName);
				}
				
				result = resourceManager.GetString(resourceName);
			}
			catch
			{
				//We tried to find a localized version of the menu name, but one was not found.
				//Default to the en-US word, which may work for the current culture.
			}

			return result;
		}

		#endregion

		#region Private Types

		private class CommandInfo
		{
			public CommandInfo(string strName, string strTooltip, int iImageIndex)
			{
				m_strName = strName;
				m_strTooltip = strTooltip;
				m_iImageIndex = iImageIndex;
			}

			public string Name { get { return m_strName; } }
			public string Tooltip { get { return m_strTooltip; } }
			public int ImageIndex { get { return m_iImageIndex; } }

			private string m_strName;
			private string m_strTooltip;
			private int m_iImageIndex;
		}

		private class MenuCommandInfo : CommandInfo
		{
			public MenuCommandInfo(string strName, string strTooltip, int iImageIndex, string strMenuName)
				: base(strName, strTooltip, iImageIndex)
			{
				m_strMenuName = strMenuName;
			}

			public string MenuName { get { return m_strMenuName; } }

			private string m_strMenuName;
		}

		#endregion

		#region Private Data Members

		private DTE2 m_Application;
		private AddIn m_AddIn;
		private CommandProcessor m_CommandProcessor;
		private string m_strCommandPrefix;

		private const string c_strToolbarName = "Menees VS Tools";

		//Note: I'm using built-in images to avoid having to ship a satellite assembly for resources.

		private static readonly CommandInfo[] s_arCommandInfo = new CommandInfo[] 
			{ 
				//These commands need text to be selected.
				new CommandInfo("Sort", "Sorts the selected lines of text", 210),
				new CommandInfo("Trim", "Trims whitespace from the selected lines of text", 20),
				new CommandInfo("Statistics", "Calculates some basic statistics for the selected lines of text", 308),
				new CommandInfo("StreamText", "Converts text with hard linebreaks into unbroken paragraphs", 1044),
				new CommandInfo("Comment", "Comments the selected text", 535),
				new CommandInfo("Uncomment", "Uncomments the selected text", 536),
				new CommandInfo("CheckSpelling", "Checks the spelling of the selected text", 2),
				new CommandInfo("AddRegion", "Surrounds the current selection with a #region", 288),
				new CommandInfo("RunText", "Executes the selected lines of text", 527), //527 346

				//These commands don't require text to be selected.
				new CommandInfo("ExecuteFile", "Executes the current file with its associated program", 144),
				new CommandInfo("GeneratePrime", "Generates a prime number", 127), //127 2384
				new CommandInfo("GenerateGUID", "Generates a new GUID and pastes it in the selection", 387),
				new CommandInfo("ToggleFiles", "Switches between related .cpp and .h files", 173),
				new CommandInfo("CloseAllExcept", "Closes all documents except the active one", 215),
				new CommandInfo("ToggleReadOnly", "Toggles the active document's read-only attribute", 277), //277 225

				//These are custom menu items.
				new MenuCommandInfo("ViewBaseConverter", "Base Converter", 7723, "View"), //7723
			};

		#endregion
	}
}