#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Copyright  2005 Bill Menees.  All rights reserved.
	Bill@Menees.com

	$History: InputDlg.cs $
	
	*****************  Version 1  *****************
	User: Bill         Date: 11/28/07   Time: 9:23p
	Created in $/CSharp/MeneesVSTools/2008
	
	*****************  Version 2  *****************
	User: Bill         Date: 5/08/05    Time: 9:33a
	Updated in $/CSharp/MeneesVSTools/2005
	
	*****************  Version 1  *****************
	User: Bill         Date: 5/01/05    Time: 9:57a
	Created in $/CSharp/MeneesVSTools/2005

-----------------------------------------------------------------------------*/

#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

#endregion

namespace Menees.VSTools
{
	public partial class InputDlg : Form
	{
		#region Constructors

		public InputDlg()
		{
			InitializeComponent();
		}

		#endregion

		#region Public Methods

		public bool GetValue(string strMessage, string strCaption, int iMinValue, int iMaxValue, ref int iValue)
		{
			bool bResult = false;

			m_iMinValue = iMinValue;
			m_iMaxValue = iMaxValue;

			Text = strCaption;
			lblPrompt.Text = strMessage;
			if (iValue != 0)
			{
				edtText.Text = iValue.ToString();
			}

			UpdateControlStates();

			if (ShowDialog() == DialogResult.OK)
			{
				//All conditions have been met so clear the error info.
				Error.SetError(edtText, "");
				bResult = ValidateInput(out iValue);
			}

			return bResult;
		}

		#endregion

		#region Private Methods

		private bool ValidateInput(out int iValue)
		{
			bool bResult = true;

			try
			{
				iValue = int.Parse(edtText.Text);
				if (iValue < m_iMinValue || iValue > m_iMaxValue)
				{
					string strMessage = string.Format("The input must be between {0} and {1}.", m_iMinValue, m_iMaxValue);
					throw new ArgumentOutOfRangeException(strMessage);
				}
			}
			catch (Exception ex)
			{
				iValue = 0;
				bResult = false;
				edtText.SelectAll();
				Error.SetError(edtText, ex.Message);
			}

			return bResult;
		}

		private void btnOK_Click(object sender, System.EventArgs e)
		{
			int iValue;
			if (!ValidateInput(out iValue))
			{
				DialogResult = DialogResult.None;
			}
		}

		private void edtText_TextChanged(object sender, EventArgs e)
		{
			UpdateControlStates();
		}

		private void UpdateControlStates()
		{
			btnOK.Enabled = edtText.Text.Trim().Length > 0;
		}

		private void edtText_KeyPress(object sender, KeyPressEventArgs e)
		{
			char ch = e.KeyChar;
			if (!char.IsControl(ch) && !char.IsDigit(ch))
			{
				e.Handled = true;
			}
		}

		#endregion

		#region Private Data Members

		private int m_iMinValue;
		private int m_iMaxValue;

		#endregion
	}
}