#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Copyright  2005 Bill Menees.  All rights reserved.
	Bill@Menees.com

	$History: LineList.cs $
	
	*****************  Version 1  *****************
	User: Bill         Date: 11/28/07   Time: 9:23p
	Created in $/CSharp/MeneesVSTools/2008
	
	*****************  Version 1  *****************
	User: Bill         Date: 5/01/05    Time: 9:57a
	Created in $/CSharp/MeneesVSTools/2005

-----------------------------------------------------------------------------*/

#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;

#endregion

namespace Menees.VSTools
{
	internal class LineList
	{
		#region Constructors

		public LineList(string strText)
		{
			SetLines(strText);
		}

		#endregion

		#region Public Methods

		public override string ToString()
		{
			if (m_arLines == null)
			{
				return "";
			}

			StringBuilder SB = new StringBuilder();

			//Convert the lines into a single string
			int iNumLines = m_arLines.Length;
			for (int i = 0; i < iNumLines; i++)
			{
				if (i != 0)
				{
					SB.Append("\r\n");
				}

				SB.Append(m_arLines[i]);
			}

			if (m_bEndsInHardReturn)
			{
				SB.Append("\r\n");
			}

			string strResult = SB.ToString();
			return strResult;
		}

		public void SetLines(string strText)
		{
			m_strText = strText;

			//Get rid of any CRs so we can use Split().
			strText = strText.Replace("\r\n", "\n");
			strText = strText.Replace("\r", "\n");

			//If the text ends with a hard return, make a note of it
			//and take it off.  This makes some of the other methods
			//easier (e.g. Sort and Comment).
			m_bEndsInHardReturn = strText.EndsWith("\n");
			if (m_bEndsInHardReturn)
			{
				strText = strText.Substring(0, strText.Length - 1);
			}

			//Split the lines
			char[] arSeparator = new Char[] { '\n' };
			m_arLines = strText.Split(arSeparator);
		}

		public void Sort(bool bCaseSensitive, bool bCompareOrdinal, bool bAscending, bool bIgnoreWS, bool bEliminateDuplicates)
		{
			GeneralStringComparer Comparer = new GeneralStringComparer(bCaseSensitive, bCompareOrdinal, bAscending, bIgnoreWS);
			Array.Sort(m_arLines, Comparer);

			if (bEliminateDuplicates)
			{
				int iNumLines = m_arLines.Length;
				List<string> arNewLines = new List<string>(iNumLines);
				string strPreviousLine = null;
				for (int i = 0; i < iNumLines; i++)
				{
					string strLine = m_arLines[i];
					if (strPreviousLine == null || Comparer.Compare(strLine, strPreviousLine) != 0)
					{
						arNewLines.Add(strLine);
					}
					strPreviousLine = strLine;
				}

				m_arLines = arNewLines.ToArray();
			}
		}

		public void Trim(bool bLeft, bool bRight)
		{
			int iNumLines = m_arLines.Length;
			for (int i = 0; i < iNumLines; i++)
			{
				if (bLeft && bRight)
				{
					m_arLines[i] = m_arLines[i].Trim();
				}
				else if (bLeft)
				{
					m_arLines[i] = m_arLines[i].TrimStart();
				}
				else if (bRight)
				{
					m_arLines[i] = m_arLines[i].TrimEnd();
				}
			}
		}

		public string Stream()
		{
			StringBuilder SB = new StringBuilder();

			int iNumLines = m_arLines.Length;
			bool bAtStartOfLine = true;
			for (int i = 0; i < iNumLines; i++)
			{
				string strLine = m_arLines[i];

				//Format the line first
				FormatLine(ref strLine);

				if (strLine.Length == 0)
				{
					SB.Append("\r\n");
					bAtStartOfLine = true;
				}
				else
				{
					if (bAtStartOfLine)
					{
						if (SB.Length > 0)
						{
							SB.Append("\r\n");
						}
					}
					else
					{
						SB.Append(" ");
					}

					SB.Append(strLine);
					bAtStartOfLine = false;
				}
			}

			if (m_bEndsInHardReturn)
			{
				SB.Append("\r\n");
			}

			string strResult = SB.ToString();
			return strResult;
		}

		public void Comment(bool bComment, string strSingle)
		{
			int iNumLines = m_arLines.Length;
			for (int i = 0; i < iNumLines; i++)
			{
				string strLine = m_arLines[i];

				if (bComment)
				{
					strLine = strSingle + strLine;
				}
				else if (strLine.StartsWith(strSingle))
				{
					strLine = strLine.Substring(strSingle.Length);
				}

				m_arLines[i] = strLine;
			}
		}

		public void Comment(bool bComment, string strStart, string strFinish)
		{
			if (m_arLines.Length == 0) return;

			if (bComment)
			{
				//Add the comment delimiters
				if (m_arLines.Length > 1)
				{
					string strNewText = strStart + "\r\n" + m_strText;
					if (m_bEndsInHardReturn)
						strNewText += strFinish + "\r\n";
					else
						strNewText += "\r\n" + strFinish;

					SetLines(strNewText);
				}
				else
				{
					m_arLines[0] = strStart + m_arLines[0] + strFinish;
				}
			}
			else
			{
				List<string> arNewLines = new List<string>(m_arLines);

				//Remove the comment delimiters
				string strLine = arNewLines[0];
				if (strLine == strStart)
				{
					arNewLines.RemoveAt(0);
				}
				else
				{
					ReplaceFirst(ref strLine, strStart, "");
					arNewLines[0] = strLine;
				}

				if (arNewLines.Count > 0)
				{
					int iLastLine = arNewLines.Count - 1;
					strLine = arNewLines[iLastLine];
					if (strLine == strFinish)
					{
						arNewLines.RemoveAt(iLastLine);
					}
					else
					{
						ReplaceFirst(ref strLine, strFinish, "");
						arNewLines[iLastLine] = strLine;
					}
				}

				m_arLines = arNewLines.ToArray();
			}
		}

		#endregion

		#region Private Methods

		private void ReplaceFirst(ref string strText, string strOld, string strNew)
		{
			int iOldStrPos = strText.IndexOf(strOld);
			if (iOldStrPos >= 0)
			{
				int iOldStrLen = strOld.Length;
				string strNewText = strText.Substring(0, iOldStrPos) + strNew + strText.Substring(iOldStrPos + iOldStrLen);
				strText = strNewText;
			}
		}

		private void FormatLine(ref string strLine)
		{
			//Strip outer whitespace first
			strLine = strLine.Trim();

			//Trim off any leading > chars that stupid email software puts in replies
			while (true)
			{
				if (strLine.Length > 0 && strLine[0] == '>')
				{
					strLine = strLine.Substring(1);
					strLine = strLine.TrimStart();
				}
				else
				{
					break;
				}
			}
		}

		#endregion

		#region Private Data Members

		private string[] m_arLines;
		private string m_strText;
		bool m_bEndsInHardReturn;

		#endregion
	}
}
