#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Copyright  2005 Bill Menees.  All rights reserved.
	Bill@Menees.com

	$History: ListDlg.cs $
	
	*****************  Version 1  *****************
	User: Bill         Date: 11/28/07   Time: 9:23p
	Created in $/CSharp/MeneesVSTools/2008
	
	*****************  Version 1  *****************
	User: Bill         Date: 5/08/05    Time: 9:33a
	Created in $/CSharp/MeneesVSTools/2005

-----------------------------------------------------------------------------*/

#endregion

#region Using Directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

#endregion

namespace Menees.VSTools
{
	public partial class ListDlg : Form
	{
		#region Constructors

		public ListDlg()
		{
			InitializeComponent();
		}

		#endregion

		#region Public Methods

		public bool Execute(string strTitle, string strPrompt, string strItemName, string[] arItems, ref string strResult)
		{
			Text = strTitle;
			lblPrompt.Text = strPrompt;
			colItem.Text = strItemName;
			edtName.Text = strResult;

			int iNumItems = arItems.Length;
			for (int i = 0; i < iNumItems; i++)
			{
				ListViewItem Item = new ListViewItem(Convert.ToString(i+1));
				Item.SubItems.Add(arItems[i]);
				lstItems.Items.Add(Item);
			}

			UpdateControlStates();

			if (ShowDialog() == DialogResult.OK)
			{
				strResult = TranslateName(edtName.Text, arItems);
				return true;
			}

			return false;
		}

		#endregion

		#region Private Helper Methods

		private void UpdateControlStates()
		{
			btnOK.Enabled = edtName.Text.Trim().Length > 0;
		}

		private string TranslateName(string strName, string[] arItems)
		{
			strName = strName.Trim();
			foreach (char ch in strName)
			{
				if (!char.IsDigit(ch))
				{
					//If we hit a non-digit, then we don't have a number.
					return strName;
				}
			}

			int iNumber = int.Parse(strName) - 1;
			if (iNumber >= 0 && iNumber < arItems.Length)
			{
				strName = arItems[iNumber];
			}

			return strName;
		}

		#endregion

		#region Private Event Handler Methods

		private void lstItems_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (lstItems.SelectedItems.Count > 0)
			{
				edtName.Text = lstItems.SelectedItems[0].SubItems[1].Text;
			}
			else
			{
				edtName.Clear();
			}
		}

		private void lstItems_DoubleClick(object sender, EventArgs e)
		{
			UpdateControlStates();
			if (lstItems.SelectedItems.Count > 0 && btnOK.Enabled)
			{
				DialogResult = DialogResult.OK;
			}
		}

		private void edtName_TextChanged(object sender, EventArgs e)
		{
			UpdateControlStates();
		}

		private void ListDlg_Load(object sender, EventArgs e)
		{
			//To size correctly, we have to do it after the window handle is created.
			//So we can't do this in the Execute method.
			lstItems.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize);
		}

		#endregion
	}
}