#region Copyright And Revision History

/*---------------------------------------------------------------------------

	Copyright  2005 Bill Menees.  All rights reserved.
	Bill@Menees.com

	$History: Utilities.cs $
	
	*****************  Version 1  *****************
	User: Bill         Date: 11/28/07   Time: 9:23p
	Created in $/CSharp/MeneesVSTools/2008
	
	*****************  Version 2  *****************
	User: Bill         Date: 5/08/05    Time: 9:33a
	Updated in $/CSharp/MeneesVSTools/2005
	
	*****************  Version 1  *****************
	User: Bill         Date: 5/01/05    Time: 9:57a
	Created in $/CSharp/MeneesVSTools/2005

-----------------------------------------------------------------------------*/

#endregion

#region Using Directives

using System;
using System.Diagnostics;
using System.Windows.Forms;
using System.Reflection;
using System.IO;
using VS = EnvDTE;

#endregion

namespace Menees.VSTools
{
	internal static class Utilities
	{
		#region Public Methods

		public static bool ExecuteCmd(string strCmd, string strPath)
		{
			Process Proc = new Process();
			ProcessStartInfo SI = Proc.StartInfo;
			SI.UseShellExecute = true;
			SI.ErrorDialog = true;
			SI.FileName = strCmd;
			SI.WorkingDirectory = strPath;
			try
			{
				return Proc.Start();
			}
			catch (Exception)
			{
				//When used from the Options dialog, if the Doc
				//file isn't found, an exception gets thrown.
				//VS.NET doesn't catch it, so if we don't then
				//the whole IDE shuts down.  That's bad...
				return false;
			}
		}

		public static bool IsShiftPressed()
		{
			//We have to use Keys.Shift instead of Keys.ShiftKey because "Shift"
			//is "The SHIFT modifier key", and it's what ModifierKeys returns.
			return (Control.ModifierKeys & Keys.Shift) == Keys.Shift;
		}

		public static int GetNextPrime(int iNearestNum)
		{
			int iResult = 0;

			if (iNearestNum >= 0)
			{
				iResult = iNearestNum;

				if (iResult <= 2)
				{
					return 2;
				}

				//Get rid of even numbers first.
				if (iResult % 2 == 0)
				{
					iResult++;
				}

				while (true)
				{
					//We only have to check the divisors up to the square root
					int iMaxDivisor = (int)(Math.Sqrt(iResult));

					bool bPrime = true;

					//Only check the odd divisors
					for (int iDivisor = 3; iDivisor <= iMaxDivisor; iDivisor += 2)
					{
						if (iResult % iDivisor == 0)
						{
							bPrime = false;
							break;
						}
					}

					if (bPrime)
					{
						break;
					}

					//Go to the next odd number
					iResult += 2;
				}
			}

			return iResult;
		}

		public static void OpenDocument(string strDocument, bool bAddDLLPath)
		{
			string strCmd = strDocument;
			string strPath = "";

			if (bAddDLLPath)
			{
				strPath = GetDllPath();
				strCmd = Path.Combine(strPath, strCmd);
			}

			ExecuteCmd(strCmd, strPath);
		}

		public static bool IsEmpty(string strText)
		{
			return strText == null || strText.Length == 0;
		}

		public static Language GetLanguage(VS.Document Doc)
		{
			//IDE languages include:
			//	C/C++
			//	HTML
			//	XML
			//	Basic
			//	CSharp
			//	CSS
			//	Plain Text
			//	PL/SQL
			//	SQL
			//	T-SQL
			//	T-SQL7
			//	T-SQL80
			//
			//Unfortunately, the IDE lumps IDL and ODL in as C/C++,
			//and VBScript and JScript in as HTML.  So we'll
			//work around that by examining the file extensions.
			//I'm also going to lump all SQL dialects into a single language.
			
			Language eResult = Language.Unknown;
			string strLanguage = Doc.Language;
			switch (strLanguage)
			{
				case "C/C++":
					eResult = Language.CPlusPlus;
					switch (Path.GetExtension(Doc.FullName).ToLower())
					{
						case ".idl":
						case ".odl":
							eResult = Language.IDL;
							break;
					}
					break;
				case "HTML":
					eResult = Language.HTML;
					switch (Path.GetExtension(Doc.FullName).ToLower())
					{
						case ".vbs":
							eResult = Language.VBScript;
							break;
						case ".js":
							eResult = Language.JScript;
							break;
					}
					break;
				case "XML":
					eResult = Language.XML;
					break;
				case "Basic":
					eResult = Language.VB;
					break;
				case "CSharp":
					eResult = Language.CSharp;
					break;
				case "CSS":
					eResult = Language.CSS;
					break;
				case "Plain Text":
					eResult = Language.PlainText;
					break;
				default: //PL/SQL, SQL, T-SQL, T-SQL7, T-SQL80, etc.
					if (strLanguage.IndexOf("SQL") >= 0)
					{
						eResult = Language.SQL;
					}
					break;
			}

			return eResult;
		}

		#endregion

		#region Private Methods

		private static string GetDllPath()
		{
			Assembly Asm = Assembly.GetExecutingAssembly();
			Module Mod = Asm.GetModule("MeneesVSTools.dll");
			if (Mod != null)
				return Path.GetDirectoryName(Mod.FullyQualifiedName);
			else
				return "";
		}

		#endregion
	}
}
